/***********************************************************
 * $Id: ComparableComparator.java 87 2007-12-17 01:06:40Z lech $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: Feb 28, 2007
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.comparators;

import java.util.Comparator;

/**
 * This class is a comparator, which compares {@link Comparable} instances
 * by forwarding the {@link Comparable#compareTo(Object)} method to the
 * implemented {@link #compareTo(Comparable)} method.  
 * 
 * @author wglas
 */
public class ComparableComparator implements Comparator
{
    private static Comparator INSTANCE = new ComparableComparator();
    
    /**
     * This is the preferred methos to acquire an instance of ComparableComparator. 
     * 
     * @return A static instance of ComparableComparator.
     */
    public static Comparator getInstance()
    {
        return INSTANCE;
    }
    
    /* (non-Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object a, Object b) {
       
        if (a == null)
            return b==null ? 0 : -1;
        
        if (!(a instanceof Comparable))
            throw new ClassCastException("First argument of class ["+a.getClass()+"] does not implement Comparable");
        
        if (b==null) return 1;
        
        return ((Comparable)a).compareTo(b);
    }

}
