/***********************************************************
 * $Id: DefaultComparatorFactory.java 87 2007-12-17 01:06:40Z lech $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: Feb 28, 2007
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.comparators;

import java.util.Comparator;

/**
 * This default implementation of ComparatorFactory simply returns
 * a ComparableComparator for Comparables.
 * 
 * @author wglas
 */
public class DefaultComparatorFactory implements ComparatorFactory {

    private static DefaultComparatorFactory INSTANCE = new DefaultComparatorFactory();
    
    /**
     * @return A static DefaultComparatorFactory instance.
     */
    public static DefaultComparatorFactory getInstance()
    {
        return INSTANCE;
    }
    
    /* (non-Javadoc)
     * @see org.clazzes.util.comparators.ComparatorFactory#getComparator(java.lang.Class)
     */
    public Comparator getComparator(Class clazz) {
        
        if (clazz.isPrimitive())
        {
            if (clazz==Void.TYPE)
                throw new ClassCastException("Cannot create a comparator for void objects.");
        }
        else if (!Comparable.class.isAssignableFrom(clazz))
            throw new ClassCastException("The supplied class ["+clazz+"] does not implement Comparable.");
            
        return ComparableComparator.getInstance();
    }

}
