/***********************************************************
 * $Id$
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 28.03.2007
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.sec;

import java.util.Collection;
import java.util.TreeMap;

/**
 * @author lech
 *
 * This factory creates password hashers.
 */
public class HasherFactory implements PasswordHasherFactory {

    public TreeMap<String, PasswordHasher> hashers_byname;

    /**
     * 
     */
    public HasherFactory() {
      checkHashers();
    }

    /**
     * Creates the hasher cache. 
     */
    private synchronized void checkHashers()
    {
        if (this.hashers_byname != null)
            return;
        this.hashers_byname = new TreeMap<String, PasswordHasher>();
        PasswordHasher plain_hasher = new PlainPasswordHasher();
        this.hashers_byname.put(plain_hasher.getAlgorithmName().toUpperCase(), plain_hasher);
        SSHA1PasswordHasher sha1_hasher = new SSHA1PasswordHasher();
        this.hashers_byname.put(sha1_hasher.getAlgorithmName().toUpperCase(), sha1_hasher);
        CryptPasswordHasher crypt_hasher = new CryptPasswordHasher();
        this.hashers_byname.put(crypt_hasher.getAlgorithmName().toUpperCase(), crypt_hasher);
    }

    /* (non-Javadoc)
     * @see org.clazzes.util.sec.PasswordHasherFactory#getPasswordHasher(java.lang.String)
     */
    public PasswordHasher getPasswordHasher(String algo_name)
    {
        //checkHashers();
        if (algo_name==null)
          return null;
        PasswordHasher hasher = this.hashers_byname.get(algo_name.toUpperCase());
        return hasher;
    }
    
    /* (non-Javadoc)
     * @see org.clazzes.util.sec.PasswordHasherFactory#getKnownPasswordHashers()
     */
    public Collection<PasswordHasher> getKnownPasswordHashers() {
        // checkHashers();
        return this.hashers_byname.values();
    }


}
