package org.clazzes.util.sec;

import java.util.Collection;

/**
 * @author lech
 *
 * Classes implementing this interface provide password hasher instances.
 */
public interface PasswordHasherFactory {

    /**
     * Returns an instance of a password hasher for the given algorithm name, or null if none could be found.
     * @param algo_name Name of hasher algorithm a PasswordHasher is wished for.
     * @return a passwordhasher, if possible, null otherwise.
     */
    public abstract PasswordHasher getPasswordHasher(String algo_name);

    /**
     * @return a Collection of password hashers known to this factory.
     */
    public abstract Collection<PasswordHasher> getKnownPasswordHashers();

}