/***********************************************************
 * $Id: FieldAccessor.java 4 2006-12-26 19:16:57Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-03-13
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.sec;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import sun.misc.BASE64Encoder;

/**
 * The SSHA1 hasher
 */
public class SSHA1PasswordHasher extends PlainPasswordHasher implements
		PasswordHasher
{
    // sun.misc.BASE64Encoder is not an officially documented JAVA
    // API, but it is at least implemented by SUN and IBM, thus making it
    // the most widely available base64 engine available in the wild.
    private BASE64Encoder base64encoder;
    
	public SSHA1PasswordHasher()
	{
		super();
		this.base64encoder = new BASE64Encoder();
	}

	/* (non-Javadoc)
	 * @see org.clazzes.util.sec.PlainPasswordHasher#hashPassword(java.lang.String, java.lang.String)
	 */
	@Override
    public String hashPassword(String cleartext, String salt)
	{
		return this.algo_prefix + sha1(cleartext, salt) + salt;
	}

	/* (non-Javadoc)
	 * @see org.clazzes.util.sec.PlainPasswordHasher#checkPassword(java.lang.String, java.lang.String)
	 */
	@Override
    public boolean checkPassword(String cleartext, String hashed)
	{
		String hash_salt = stripAlgorithmKey(hashed);
		int l = hash_salt.length(); // the naked hash is 20 bytes long, but
									// base64 makes it longer!
		String salt = hash_salt.substring(l - 4);
		String rehashed = hashPassword(cleartext, salt);
		return rehashed.equals(hashed);
	}

	/* (non-Javadoc)
	 * @see org.clazzes.util.sec.PlainPasswordHasher#getAlgorithmName()
	 */
	@Override
    public String getAlgorithmName()
	{
		return "SSHA1";
	}

	private String sha1(String cleartext, String salt)
	{
		try
		{
			// convert input to utf-8 based byte[]
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			OutputStreamWriter osw = new OutputStreamWriter(baos, "UTF-8");
			osw.write(cleartext);
			osw.write(salt);
			osw.close();
			baos.close();

			// make sha1 stuff
			MessageDigest sha1digest = MessageDigest.getInstance("SHA-1");
			sha1digest.update(baos.toByteArray());
			byte[] sha1ba = sha1digest.digest();
			String s = this.base64encoder.encode(sha1ba);

			return s + salt;
		} catch (NoSuchAlgorithmException e)
		{
			throw new RuntimeException(e);
		} catch (UnsupportedEncodingException e)
		{
			throw new RuntimeException(e);
		} catch (IOException e)
		{
			throw new RuntimeException(e);
		}
	}
    
    
    /* (non-Javadoc)
     * @see org.clazzes.util.sec.PasswordHasher#getDefaultSaltLength()
     */
    public int getSaltLength() {
        return 4;
    }


}
