/***********************************************************
 * $Id: ThreadStopper.java 87 2007-12-17 01:06:40Z lech $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-12-27
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.thread;

/**
 * @author wglas
 * 
 * this class is used in threads, which periodically check for an external
 * request for terminating it's work.
 */
public class ThreadStopper {
    /**
     * This variable is set to true, if the thread has been requested to finish.
     */
    boolean toBeFinished;

    /**
     * Instantiates a new ThreadStopper instance, which is in the
     * not-being-stopped state.
     */
    public ThreadStopper() {
        super();
        this.toBeFinished = false;
    }

    /**
     * This method is used inside the thread's main loop in order to check,
     * whether the thread has be stopped the next time it has finished a portion
     * of his work.
     * 
     * @return true, if we should leave the thread loop now.
     */
    public synchronized boolean isToBeFinished() {
        return this.toBeFinished;
    }

    /**
     * This function waits up to the given timeout in milliseconds or exits
     * immediately after requestStop() has been called. In the later case, the
     * function returns true.
     * 
     * @param timeout
     *            The maximal number of milliseconds to wait.
     * @return true, if the main loop should be left immediately.
     * @throws InterruptedException
     *             if some other thread calls Thread.interrupt on this Thread
     *             while waiting.
     */
    public synchronized boolean waitToBeFinished(long timeout)
            throws InterruptedException {
        if (!this.toBeFinished)
            this.wait(timeout);

        return this.toBeFinished;
    }

    public synchronized void waitToBeFinished() throws InterruptedException {
        this.wait();
    }

    /**
     * Requests the thread to be stopped. Since the thread may just be doing
     * some work, it may take some time until the thread really stops.
     */
    public synchronized void requestStop() {
        this.toBeFinished = true;
        this.notifyAll();
    }

    /**
     * Requests the thread to be stopped. Since the thread may just be doing
     * some work, it may take some time until the thread really stops.
     */
    public synchronized void resume() {
        this.notifyAll();
    }
}
